     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.util.param;

     import java.io.*;
     import java.net.*;
     import java.util.*;

     import com.orbs.open1.mit.lang.StringLib;

     import com.orbs.open1.mpl.j1d1.net.URLCoder;

     /*********************************************************************
     *
     * Default methods for classes that implement the Parameters interface
     * and utility methods.
     *
     * <P>
     *
     * @see
     *   Parameters
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   2000-04-28
     *********************************************************************/

     public class  ParametersLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  ParametersLib ( ) { }

     /*********************************************************************
     * Adds the <I>parameterName</I> with an empty array of parameter
     * values.
     *
     * @return
     *   Returns false if the name already exists.
     * @exception  IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public static boolean  addName (
       Hashtable  hashtable,
       String     parameterName )
     //////////////////////////////////////////////////////////////////////
     {
       if ( parameterName == null )
       {
         throw new IllegalArgumentException ( "null parameterName" );
       }

       if ( hashtable.get ( parameterName ) != null ) return false;

       hashtable.put ( parameterName, new String [ ] { } );

       return true;
     }

     /*********************************************************************
     * Returns the first value of the specified parameter name.
     * Returns null if the parameter name or value does not exist.
     * <P>
     * Relies upon a call to parameters.getValues(String).
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public static String  getFirstValue (
       Parameters  parameters, String  parameterName )
     //////////////////////////////////////////////////////////////////////
     {
       String [ ]  parameterValues
         = parameters.getValues ( parameterName );
       if ( ( parameterValues == null    )
         || ( parameterValues.length < 1 ) )
       {
         return null;
       }
       return parameterValues [ 0 ];
     }

     /*********************************************************************
     * Returns the first value of the specified parameter name.
     * Returns <I>defaultValue</I> if the parameter name or value does not
     * exist.
     * <P>
     * Relies upon a call to parameters.getValue(String).
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public static String  getFirstValue (
       Parameters  parameters,
       String      parameterName,
       String      defaultValue )
     //////////////////////////////////////////////////////////////////////
     {
       String  parameterValue = parameters.getFirstValue ( parameterName );
       if ( parameterValue == null ) return defaultValue;
       return parameterValue;
     }

     /*********************************************************************
     * Returns a String array of the parameter names.
     * Returns an empty array if there are no parameter names.
     *********************************************************************/
     public static String [ ]  getNames ( Hashtable  hashtable )
     //////////////////////////////////////////////////////////////////////
     {
       Enumeration  keyEnumeration = hashtable.keys ( );

       String [ ]  names = new String [ hashtable.size ( ) ];

       for ( int  i = 0; i < names.length; i++ )
       {
         names [ i ] = ( String ) keyEnumeration.nextElement ( );
       }

       return names;
     }

     /*********************************************************************
     * Returns a String array of values of the specified parameter name.
     * Returns null if the named parameter does not exist.
     * Returns an empty array if no values exist for the name.
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public static String [ ]  getValues (
       Hashtable  hashtable, String  parameterName )
     //////////////////////////////////////////////////////////////////////
     {
       if ( parameterName == null )
       {
         throw new IllegalArgumentException ( "null parameterName" );
       }

       return ( String [ ] ) hashtable.get ( parameterName );
     }

     public static void  list ( Parameters  parameters, PrintStream  out )
     //////////////////////////////////////////////////////////////////////
     {
       if ( parameters == null ) return;

       String [ ]  parameterNames = parameters.getNames ( );
       if ( parameterNames == null ) return;

       for ( int  i = 0; i < parameterNames.length; i++ )
       {
         String [ ]  parameterValues
           = parameters.getValues ( parameterNames [ i ] );
         if ( parameterValues.length < 1 )
         {
           out.println ( parameterNames [ i ] );
         }
         else
         {
           for ( int  j = 0; j < parameterValues.length; j++ )
           {
             out.println (
               parameterNames [ i ] + "=" + parameterValues [ j ] );
           }
         }
       }
     }

     /*********************************************************************
     * Removes the <I>parameterName</I> from this container.
     *
     * @return
     *   The parameter values or null if the <I>parameterName</I> could not
     *   be removed.
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public static String [ ]  removeName (
       Hashtable  hashtable, String  parameterName )
     //////////////////////////////////////////////////////////////////////
     {
       return ( String [ ] ) hashtable.remove ( parameterName );
     }

     /*********************************************************************
     * Sets the <I>parameterValues</I> for the given <I>parameterName</I>.
     *
     * @param  parameterValues
     *   May be an empty array of String.
     * @return
     *   The previous set of parameter values.
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> or <I>parameterValues</I> is null.
     *********************************************************************/
     public static String [ ]  setValues (
       Hashtable   hashtable,
       String      parameterName,
       String [ ]  parameterValues )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( parameterName == null ) || ( parameterValues == null ) )
       {
         throw new IllegalArgumentException ( "null parameterName" );
       }

       return ( String [ ] ) hashtable.put (
         parameterName, parameterValues );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Parses the Parameters out of a URL-encoded HTTP "get" String.
     *
     * @param data
     *   The URL encoded parameters starting just after the question mark
     *   from an HTTP "get" request string.
     * @return
     *   The Parameters with the names and values decoded and the names
     *   trimmed.
     *********************************************************************/
     public static Parameters  parseHttpGetParameters ( String  data )
     //////////////////////////////////////////////////////////////////////
     {
       Hashtable  hashtable = new Hashtable ( );

       StringTokenizer  stringTokenizer
         = new StringTokenizer ( data, "&" );

       while ( stringTokenizer.hasMoreElements ( ) )
       {
         String  token = stringTokenizer.nextToken ( );

         String [ ]  nameValuePair
           = StringLib.splitAtDelimiter ( token, "=" );

         String  name  = nameValuePair [ 0 ];

         String  value = nameValuePair [ 1 ];

         try
         {
           if ( name != null )
           {
             name = URLCoder.decode ( name ).trim ( );
           }
           else
           {
             throw new IllegalArgumentException (
               "bad parameters:  " + data );
           }

           if ( value != null )
           {
             value = URLCoder.decode ( value );
           }

           Vector  vector = ( Vector ) hashtable.get ( name );

           if ( vector == null )
           {
             vector = new Vector ( );

             hashtable.put ( name, vector );
           }

           if ( value != null ) vector.addElement ( value );
         }
         catch ( Exception  ex )
         {
           ex.printStackTrace ( );
         }
       }

       Parameters  parameters = new ParametersHashtable ( );

       Enumeration  enumeration = hashtable.keys ( );

       while ( enumeration.hasMoreElements (  ) )
       {
         String  key = ( String ) enumeration.nextElement ( );

         Vector  vector = ( Vector ) hashtable.get ( key );

         String [ ]  values = new String [ vector.size ( ) ];

         for ( int  j = 0; j < values.length; j++ )
         {
           values [ j ] = ( String ) vector.elementAt ( j );
         }

         parameters.setValues ( key, values );
       }

       return parameters;
     }

     public static String  urlEncode ( Parameters  parameters )
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  buf = new StringBuffer ( );

       String [ ]  names = parameters.getNames ( );

       for ( int  i = 0; i < names.length; i++ )
       {
         String [ ]  values = parameters.getValues ( names [ i ] );

         if ( ( values != null    )
           && ( values.length > 0 ) )
         {
           for ( int  j = 0; j < values.length; j++ )
           {
             buf.append (
               ( ( ( i == 0 ) && ( j == 0 ) ) ? "" : "&" )
               + names [ i ] + "="
               + URLEncoder.encode ( values [ j ] ) );
           }
         }
         else
         {
           buf.append ( ( ( i == 0 ) ? "" : "&" ) + names [ i ] );
         }
       }

       return buf.toString ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
