     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.queue.ListQueue",
     // created 1999-02-07.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.j1d2.util.queue;

     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.Queue;

     /*********************************************************************
     * An implementation of Queue that relies upon a List backbone.
     * The methods synchronize on the List where necessary.
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   2000-04-20
     *********************************************************************/

     public class  ListQueue implements Queue
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     protected List  list;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ListQueue ( List  list )
     //////////////////////////////////////////////////////////////////////
     {
       this.list = list;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Appends the object to the queue and notifies any threads that
     * are blocked in pull().
     *
     * @return
     *   Returns true if the object could be added to the queue.
     *********************************************************************/
     public boolean  append ( Object  o )
     //////////////////////////////////////////////////////////////////////
     {
       if ( o == null )
       {
         throw new IllegalArgumentException ( "null" );
       }

       if ( list.add ( o ) )
       {
         synchronized ( list )
         {
           list.notifyAll ( );
         }
         return true;
       }
       else
       {
         return false;
       }
     }

     /*********************************************************************
     * Poll this queue to see if an object is available, immediately
     * removing one if so.  If the queue is empty, this method
     * immediately returns null.
     *
     * @return
     *   An object, if one was immediately available, otherwise null.
     *********************************************************************/
     public Object  poll ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         return list.remove ( 0 );
       }
       catch ( IndexOutOfBoundsException  ex )
       {
         return null;
       }
     }

     /*********************************************************************
     * Remove the next object in this queue, blocking until one becomes
     * available.
     *
     * @throws InterruptedException
     *    If the wait is interrupted.
     *********************************************************************/
     public Object  pull ( ) throws InterruptedException
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         return pull ( 0 );
       }
       catch ( IllegalArgumentException  ex )
       {
         throw new RuntimeException ( ex.getMessage ( ) );
       }
     }

     /*********************************************************************
     * Remove the next object in this queue, blocking until one becomes
     * available or the given timeout period expires.
     *
     * @param timeout
     *   If positive, block for up to <I>timeout</I> milliseconds while
     *   waiting for an object to be added to this queue.
     *   If zero, block indefinitely.
     *
     * @return
     *   An object, if one was available within the specified timeout
     *   period, otherwise null.
     *
     * @throws IllegalArgumentException
     *    If the value of the timeout argument is negative.
     * @throws InterruptedException
     *    If the timeout wait is interrupted.
     *********************************************************************/
     public Object  pull ( long  timeout )
       throws IllegalArgumentException, InterruptedException
     //////////////////////////////////////////////////////////////////////
     // Untested!!!
     //////////////////////////////////////////////////////////////////////
     {
       if ( timeout < 0 )
       {
         throw new IllegalArgumentException ( "timeout < 0" );
       }

       long  stopTime = new Date ( ).getTime ( ) + timeout;

       Object  o = null;

       while ( ( o = poll ( ) ) == null )
       {
         if ( timeout == 0 )
         {
           synchronized ( list )
           {
             list.wait ( );
           }
         }
         else
         {
           long  now = new Date ( ).getTime ( );

           if ( stopTime > now )
           {
             synchronized ( list )
             {
               list.wait ( stopTime - now );
             }
           }
           else
           {
             break;
           }
         }
       }

       return o;
     }

     /*********************************************************************
     * Replaces the first occurrence of any equal object in the queue.
     * The new object is placed in the same queue order as the old.
     * If the new object is unique, calls append(o).
     *
     * @return
     *   The old object, if available, otherwise null.
     *********************************************************************/
     public Object  replace ( Object  o )
     //////////////////////////////////////////////////////////////////////
     {
       synchronized ( list )
       {
         int  index = list.indexOf ( o );

         if ( index < 0 )
         {
           append ( o );
           return null;
         }
         else
         {
           return list.set ( index, o );
         }
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
